@@ -126,9 +126,9 @@ def all_plugin_dirs(name):
         for l in os.listdir(d):
             n = '%s/%s/plugins/%s_%s' % (d, l, name, l)
             if exists(n):
-                name = os.path.basename(n)
-                if not name in all:
-                    all[name] = n
+                bname = os.path.basename(n)
+                if not bname in all:
+                    all[bname] = n
 
     [scan_sys_pth(d) for d in sys.path if os.path.isdir(d)]
     return all.values()
