@@ -103,7 +103,7 @@ def flatten(d, sep='_'):
     return obj
 
 
-def cast(v):
+def cast(v, bools={'true': True, 'True': True, 'false': False, 'False': False}):
     if v and v[0] in ('{', '['):
         return json.loads(v)
     try:
@@ -112,7 +112,7 @@ def cast(v):
         try:
             return float(v)
         except:
-            return {'true': True, 'false': False}.get(v, v)
+            return bools.get(v, v)
 
 
 # --------------------------------------------------------------------------------- tty
@@ -123,6 +123,11 @@ try:
     term_fixed_width_env = int(os.environ.get('term_width'))
 except:
     term_fixed_width_env = 0
+try:
+    # Setting the default for the cli - flag, i.e. this rules w/o the flag:
+    term_fixed_height_env = int(os.environ.get('term_height'))
+except:
+    term_fixed_height_env = 0
 
 
 def walk_dir(directory, crit=None):
@@ -220,9 +225,10 @@ class Pytest:
         FLG(sys.argv)  # parsing the flags
 
 
-def terminal_size():
-    import fcntl, termios, struct
+import fcntl, termios, struct
 
+
+def terminal_size():
     # try piped left and/or right, if both piped then return 80,25:
     # since we want to print we try first stdout:
     for fd in 1, 0:
@@ -231,51 +237,25 @@ def terminal_size():
                 'HHHH',
                 fcntl.ioctl(fd, termios.TIOCGWINSZ, struct.pack('HHHH', 0, 0, 0, 0)),
             )
-            return h, (w or 80)  # go really sure its no 0
+            return (w or 80), h  # go really sure its no 0
         except:
             pass
     return 80, 25
 
 
-def termwidth(c=[-1, 0]):
-    """
-    somewhat expensive, we run max every 2 seconds, i.e. potentially
-    redraw delays when term resizing
-    Right now we have to foreground term app anyway but maybe dev log rearranging
-    """
+def termwidth():
+    return termsize()[0]
+
+
+def termsize():
     try:
-        # when called before flag parseing, e.g. at -hf
-        flg_fixed, flg_dt_recalc = (
-            FLG.term_fixed_width,
-            FLG.term_auto_width_recalc_every,
-        )
-    except Exception as ex:
-        flg_fixed, flg_dt_recalc = term_fixed_width_env or None, None
-    if flg_dt_recalc:
-        if c[0] > -1 and now() - c[1] < FLG.term_auto_width_recalc_every:
-            return c[0]
-
-    def _tw():
-        tfw = flg_fixed
-        if tfw is not None and tfw > 0:
-            return tfw
-        return terminal_size()[1]
-        # try:
-        # wtmfMF?!?
-        #     # This seems to be working in the most situations - i.e. with a shell
-        #     # better than os.popen('tput cols 2>/dev/null')
-        #     os.system('stty size > /tmp/sttysize 2>/dev/null')
-        #     with open('/tmp/sttysize') as fd:
-        #         s = fd.read()
-        #     i = int(s.strip().split(' ', 1)[1])
-        #     if not i:
-        #         raise Exception('x')
-        #     return i
-        # except:
-        #     return 80
-
-    c[0], c[1] = _tw(), now()
-    return c[0]
+        w, h = FLG.term_fixed_width, FLG.term_fixed_height
+    except:
+        # not yet parsed?
+        w, h = 0, 0
+    if w and h:
+        return w, h
+    return terminal_size()
 
 
 # -------------------------------------------------------------------------- data utils
@@ -572,9 +552,7 @@ def wait_for_port(port, host='127.0.0.1', timeout=5.0, log_err=True):
                     print('timeout awaiting port %s' % port)
 
 
-def repl_dollar_var_with_env_val(
-    s, die_on_fail=True, ask_on_fail=False, get_vals=False
-):
+def repl_dollar_var_with_env_val(s, die_on_fail=True, ask_on_fail=False, get_vals=False):
     """
     s = foo_$bar.or${baz}  - the first form searched until first non Letter char
     Will be replaced by value of environ for $bar and $baz
@@ -914,9 +892,7 @@ class LazyDict(dict):
         return (
             self[key]
             if key in self
-            else dict.setdefault(
-                self, key, thunk(*a, **kw) if callable(thunk) else thunk
-            )
+            else dict.setdefault(self, key, thunk(*a, **kw) if callable(thunk) else thunk)
         )
 
 
@@ -1107,14 +1083,12 @@ def set_flag_vals_from_env():
 
 def shorten(key, prefix, maxlen, all_shorts=None, take=1):
     take, sold = 1, None
-    while True:
-        s = autoshort(key, prefix, maxlen, take)
+    for i in range(10):
+        s = autoshort(key, prefix, maxlen + i, take)
         if not s in all_shorts:
-            all_shorts.add(s)
             return s
-        if s == sold:
-            raise Exception('Unresolvable collision in autoshort names: %s' % key)
-        sold, take, maxlen = s, take + 1, maxlen + 0
+    if s == sold:
+        raise Exception('Unresolvable collision in autoshort names: %s' % key)
 
 
 def autoshort(key, prefix, maxlen, take=1):
@@ -1156,29 +1130,35 @@ def flag_makers(t, m=_flag_makers):
         return _flag_makers[t]
 
 
-all_flag_shorts = set()
+all_flag_shorts = {}
 
 
 def make_flag(c, module, autoshort, default, sub=False, **kw):
     g = getattr
-    key = c.__name__
-    if sub:
+    key = orig_key = c.__name__
+    if sub and not sub == 'Actions':
         key = sub + '_' + key
     d = g(c, 'd', default)  # when d is not given, what to do. Dictates the flag type.
     ml = kw.get('short_maxlen', 5)
-    mkw = {'module_name': module, 'short_name': g(c, 's', None)}
     s = g(c, 's', None)
+    mkw = {'module_name': module, 'short_name': s}
     if s is None and autoshort is not False:
-        s = shorten(key, prefix=autoshort, maxlen=ml, all_shorts=all_flag_shorts)
+        s = shorten(orig_key, prefix=autoshort, maxlen=ml, all_shorts=all_flag_shorts)
     if s is False:
         s = None
+    else:
+        all_flag_shorts[s] = key
+        # when we parse the cli for action flags we also want to find shortened afs:
+        af = action_flags.get(key)
+        if af:
+            action_flags[s] = af
     mkw['short_name'] = s
-    m = flag_makers(g(c, 't', type(d)))
+    define_flag = flag_makers(g(c, 't', type(d)))
     txt = g(c, 'n', human(key))
     if c.__doc__:
         ls = c.__doc__.replace(c.n, '').splitlines()
         txt += ' Details: %s' % ('\n'.join([l.strip() for l in ls]).strip())
-    m(key, d, txt, **mkw)
+    define_flag(key, d, txt, **mkw)
 
 
 human = lambda key: ' '.join([k.capitalize() for k in key.split(' ')])
@@ -1189,13 +1169,28 @@ skip_flag_defines = []
 
 
 have_flg_cls = set()
+action_flags = {}
 
 
-def define_flags(Class, sub=False):
+def have_subs(c):
+    for s in [getattr(c, s) for s in dir(c) if not s.startswith('_')]:
+        if isinstance(s, type):
+            return True
+
+
+def rm_absl_flags():
+    FLG.remove_flag_values(
+        ['v',]
+    )
+
+
+def define_flags(Class, sub=False, parent_autoshort=False):
     """
     Pretty classes to flag defs. See nrclient, devapp.app or structlogging.sl how to use
     All config in the top class
+    2021/06: Subclassses allowed
     """
+
     if Class in have_flg_cls:
         from devapp.app import app
 
@@ -1206,34 +1201,44 @@ def define_flags(Class, sub=False):
     module = g(Class, 'module', Class.__module__)
     if module in skip_flag_defines:
         return
-    autoshort = g(Class, 'autoshort', sub[0] if sub else False)
     # passed to make_flag as default for default
     default = g(Class, 'default', '')
+    autoshort = parent_autoshort = g(Class, 'autoshort', parent_autoshort)
     l = dict(locals())
-    cs, cnos = [], []
-    for k in dir(Class):
-        if k.startswith('_'):
-            continue
+    cshrt, c_no_shrt = [], []
+    for k in [i for i in dir(Class) if not i.startswith('_')]:
         c = g(Class, k)
+
         if not isinstance(c, type):
             continue
+        if sub == 'Actions':
+            setattr(c, 'd', False if not g(c, 'd', None) == True else True)
+            action_flags[k] = {
+                'flg_cls': c,
+                'class': Class,
+                'key': k,
+                'autoshort': g(c, 'autoshort', autoshort),
+                'is_default': c.d,
+            }
+
         if not hasattr(c, 'n'):
             if not hasattr(c, 'd'):
+                a = g(c, 'autoshort', parent_autoshort)
+                if not a and not c.__name__ == 'Actions':
+                    pref = c.__name__[0]
                 # prbably group but no n, no d is allowed as well
-                subs = [g(c, s) for s in dir(c) if not s.startswith('_')]
-                subs = [i for i in subs if isinstance(i, type)]
-                if subs:
-                    r = define_flags(c, sub=c.__name__)
+                if have_subs(c):
+                    r = define_flags(c, sub=c.__name__, parent_autoshort=a)
                     continue
             # alternative to stating n we allow a multline docstring, where first line
             # is n, rest is details:
             doc = c.__doc__ or ''
             if doc.strip():
                 c.n = doc.strip().split('\n', 1)[0]
-
-        cs.append(c) if g(c, 's', None) else cnos.append(c)
+        cshrt.append(c) if g(c, 's', None) else c_no_shrt.append(c)
     # do the ones with shorts first, to collide only on autoshorts:
-    [make_flag(c, **l) for k in [cs, cnos] for c in k]
+    [make_flag(c, **l) for c in cshrt + c_no_shrt]
+    # [make_flag(c, **l) for k in [cshrt, c_no_shrt] for c in k]
 
 
 # define_flags(Flags)  # ours
@@ -1372,9 +1377,6 @@ class project:
             c['project']['urls']['repository'] = c['project']['repository']
 
         t = cfg['tool']
-        # breakpoint()  # FIXME BREAKPOINT
-        # c['app'] = t['poetry']
-        # c['app'].update(t.get('lc', {}).get('app', {}))
         project.fn_cfg = fn
         # app.die('Did not find a pyproject.toml file with badges declared')
         return project.config
@@ -1574,9 +1576,14 @@ class appflags:
         n = 'Sets terminal width to this. 0: auto.'
         d = term_fixed_width_env
 
-    class term_auto_width_recalc_every:
-        n = 'When term with is auto we run stty size maximum every so many millisecs'
-        d = 2000
+    class term_fixed_height:
+        n = 'Sets terminal size to this. 0: auto.'
+        d = term_fixed_height_env
+
+    # not needed to run os.system calls anymore:
+    # class term_auto_width_recalc_every:
+    #     n = 'When term with is auto we run stty size maximum every so many millisecs'
+    #     d = 2000
 
     class help_output_fmt:
         s = False  #  must be long form, checked in absl_color_help
